@@ -123,6 +123,8 @@ struct SSConfig {
     mode: Option<String>,
     #[serde(skip_serializing_if = "Option::is_none")]
     no_delay: Option<bool>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    keep_alive: Option<u64>,
     #[cfg(all(unix, not(target_os = "android")))]
     #[serde(skip_serializing_if = "Option::is_none")]
     nofile: Option<u64>,
@@ -137,6 +139,9 @@ struct SSLocalExtConfig {
     local_address: Option<String>,
     local_port: u16,
 
+    #[serde(skip_serializing_if = "Option::is_none")]
+    disabled: Option<bool>,
+
     #[serde(skip_serializing_if = "Option::is_none")]
     mode: Option<String>,
 
@@ -250,11 +255,8 @@ impl ConfigType {
 
 cfg_if! {
     if #[cfg(feature = "local-redir")] {
-        use strum::IntoEnumIterator;
-        use strum_macros::EnumIter;
-
         /// Transparent Proxy type
-        #[derive(Clone, Copy, Debug, Eq, PartialEq, EnumIter)]
+        #[derive(Clone, Copy, Debug, Eq, PartialEq)]
         pub enum RedirType {
             /// For not supported platforms
             NotSupported,
@@ -301,44 +303,100 @@ cfg_if! {
             cfg_if! {
                 if #[cfg(any(target_os = "linux", target_os = "android"))] {
                     /// Default TCP transparent proxy solution on this platform
-                    pub fn tcp_default() -> RedirType {
+                    pub const fn tcp_default() -> RedirType {
                         RedirType::Redirect
                     }
 
+                    /// Available TCP transparent proxy types
+                    #[doc(hidden)]
+                    pub fn tcp_available_types() -> &'static [&'static str] {
+                        const AVAILABLE_TYPES: &[&str] = &[RedirType::Redirect.name(), RedirType::TProxy.name()];
+                        AVAILABLE_TYPES
+                    }
+
                     /// Default UDP transparent proxy solution on this platform
-                    pub fn udp_default() -> RedirType {
+                    pub const fn udp_default() -> RedirType {
                         RedirType::TProxy
                     }
+
+                    /// Available UDP transparent proxy types
+                    #[doc(hidden)]
+                    pub fn udp_available_types() -> &'static [&'static str] {
+                        const AVAILABLE_TYPES: &[&str] = &[RedirType::TProxy.name()];
+                        AVAILABLE_TYPES
+                    }
                 } else if #[cfg(any(target_os = "openbsd", target_os = "freebsd"))] {
                     /// Default TCP transparent proxy solution on this platform
                     pub fn tcp_default() -> RedirType {
                         RedirType::PacketFilter
                     }
 
+                    /// Available TCP transparent proxy types
+                    #[doc(hidden)]
+                    pub fn tcp_available_types() -> &'static [&'static str] {
+                        const AVAILABLE_TYPES: &[&str] = &[RedirType::PacketFilter.name(), RedirType::IpFirewall.name()];
+                        AVAILABLE_TYPES
+                    }
+
                     /// Default UDP transparent proxy solution on this platform
                     pub fn udp_default() -> RedirType {
                         RedirType::PacketFilter
                     }
+
+                    /// Available UDP transparent proxy types
+                    #[doc(hidden)]
+                    pub const fn udp_available_types() -> &'static [&'static str] {
+                        const AVAILABLE_TYPES: &[&str] = &[RedirType::PacketFilter.name(), RedirType::IpFirewall.name()];
+                        AVAILABLE_TYPES
+                    }
                 } else if #[cfg(any(target_os = "netbsd", target_os = "solaris", target_os = "macos", target_os = "ios"))] {
                     /// Default TCP transparent proxy solution on this platform
                     pub fn tcp_default() -> RedirType {
                         RedirType::PacketFilter
                     }
 
+                    /// Available TCP transparent proxy types
+                    #[doc(hidden)]
+                    pub const fn tcp_available_types() -> &'static [&'static str] {
+                        const AVAILABLE_TYPES: &[&str] = &[RedirType::PacketFilter.name(), RedirType::IpFirewall.name()];
+                        AVAILABLE_TYPES
+                    }
+
                     /// Default UDP transparent proxy solution on this platform
                     pub fn udp_default() -> RedirType {
                         RedirType::NotSupported
                     }
+
+                    /// Available UDP transparent proxy types
+                    #[doc(hidden)]
+                    pub const fn udp_available_types() -> &'static [&'static str] {
+                        const AVAILABLE_TYPES: &[&str] = &[];
+                        AVAILABLE_TYPES
+                    }
                 } else {
                     /// Default TCP transparent proxy solution on this platform
                     pub fn tcp_default() -> RedirType {
                         RedirType::NotSupported
                     }
 
+                    /// Available TCP transparent proxy types
+                    #[doc(hidden)]
+                    pub const fn tcp_available_types() -> &'static [&'static str] {
+                        const AVAILABLE_TYPES: &[&str] = &[];
+                        AVAILABLE_TYPES
+                    }
+
                     /// Default UDP transparent proxy solution on this platform
                     pub fn udp_default() -> RedirType {
                         RedirType::NotSupported
                     }
+
+                    /// Available UDP transparent proxy types
+                    #[doc(hidden)]
+                    pub const fn udp_available_types() -> &'static [&'static str] {
+                        const AVAILABLE_TYPES: &[&str] = &[];
+                        AVAILABLE_TYPES
+                    }
                 }
             }
 
@@ -348,7 +406,7 @@ cfg_if! {
             }
 
             /// Name of redirect type (transparent proxy type)
-            pub fn name(self) -> &'static str {
+            pub const fn name(self) -> &'static str {
                 match self {
                     // Dummy, shouldn't be used in any useful situations
                     RedirType::NotSupported => "not_supported",
@@ -373,19 +431,6 @@ cfg_if! {
                     RedirType::IpFirewall => "ipfw",
                 }
             }
-
-            /// Get all available types
-            pub fn available_types() -> Vec<&'static str> {
-                let mut v = Vec::new();
-                for e in Self::iter() {
-                    match e {
-                        RedirType::NotSupported => continue,
-                        #[allow(unreachable_patterns)]
-                        _ => v.push(e.name()),
-                    }
-                }
-                v
-            }
         }
 
         impl Display for RedirType {
@@ -726,6 +771,13 @@ pub struct Config {
     pub no_delay: bool,
     /// Set `TCP_FASTOPEN` socket option
     pub fast_open: bool,
+    /// Set TCP Keep-Alive duration, will set both `TCP_KEEPIDLE` and `TCP_KEEPINTVL`
+    ///
+    /// https://github.com/shadowsocks/shadowsocks-rust/issues/546
+    ///
+    /// If this is not set, sockets will be set with a default timeout
+    pub keep_alive: Option<Duration>,
+
     /// `RLIMIT_NOFILE` option for *nix systems
     #[cfg(all(unix, not(target_os = "android")))]
     pub nofile: Option<u64>,
@@ -840,6 +892,8 @@ impl Config {
 
             no_delay: false,
             fast_open: false,
+            keep_alive: None,
+
             #[cfg(all(unix, not(target_os = "android")))]
             nofile: None,
 
@@ -958,6 +1012,10 @@ impl Config {
                 // `locals` are only effective in local server
                 if let Some(locals) = config.locals {
                     for local in locals {
+                        if local.disabled.unwrap_or(false) {
+                            continue;
+                        }
+
                         if local.local_port == 0 {
                             let err = Error::new(ErrorKind::Malformed, "`local_port` cannot be 0", None);
                             return Err(err);
@@ -1107,7 +1165,7 @@ impl Config {
 
         // Standard config
         // Server
-        match (config.server, config.server_port, config.password, config.method) {
+        match (config.server, config.server_port, config.password, &config.method) {
             (Some(address), Some(port), Some(pwd), Some(m)) => {
                 let addr = match address.parse::<Ipv4Addr>() {
                     Ok(v4) => ServerAddr::SocketAddr(SocketAddr::V4(SocketAddrV4::new(v4, port))),
@@ -1151,6 +1209,9 @@ impl Config {
 
                 nconfig.server.push(nsvr);
             }
+            (None, None, None, Some(_)) if config_type.is_manager() => {
+                // Set the default method for manager
+            }
             (None, None, None, None) => (),
             _ => {
                 let err = Error::new(
@@ -1286,6 +1347,21 @@ impl Config {
 
             let mut manager_config = ManagerConfig::new(manager);
             manager_config.mode = global_mode;
+
+            if let Some(ref m) = config.method {
+                match m.parse::<CipherKind>() {
+                    Ok(method) => manager_config.method = Some(method),
+                    Err(..) => {
+                        let err = Error::new(
+                            ErrorKind::Invalid,
+                            "unsupported method",
+                            Some(format!("`{}` is not a supported method", m)),
+                        );
+                        return Err(err);
+                    }
+                }
+            }
+
             nconfig.manager = Some(manager_config);
         }
 
@@ -1309,6 +1385,11 @@ impl Config {
             nconfig.fast_open = b;
         }
 
+        // TCP Keep-Alive
+        if let Some(d) = config.keep_alive {
+            nconfig.keep_alive = Some(Duration::from_secs(d));
+        }
+
         // UDP
         nconfig.udp_timeout = config.udp_timeout.map(Duration::from_secs);
 
@@ -1609,6 +1690,7 @@ impl fmt::Display for Config {
                             ServerAddr::SocketAddr(ref sa) => sa.port(),
                             ServerAddr::DomainName(.., port) => port,
                         },
+                        disabled: None,
                         local_udp_address: local.udp_addr.as_ref().map(|udp_addr| match udp_addr {
                             ServerAddr::SocketAddr(sa) => sa.ip().to_string(),
                             ServerAddr::DomainName(dm, ..) => dm.to_string(),
@@ -1798,6 +1880,10 @@ impl fmt::Display for Config {
             jconf.fast_open = Some(self.fast_open);
         }
 
+        if let Some(keepalive) = self.keep_alive {
+            jconf.keep_alive = Some(keepalive.as_secs());
+        }
+
         match self.dns {
             DnsConfig::System => {}
             #[cfg(feature = "trust-dns")]
